<?php

namespace Service;


use Doctrine\ORM\EntityManager;

class Profile {

    protected  $entityManager;
    protected  $environment;
    protected  $profileFactory;

    const EMPLOYEE_CATEGORY = 'E';
    const SPONSOR_CATEGORY = 'S';

    const ADMINISTRATOR_LABEL =     'Administrador';
    const ADMINISTRATOR_CONTROL =   'A';
    const PROJECT_MANAGER_LABEL =   'Gerente de Projetos';
    CONST PROJECT_MANAGER_CONTROL = 'M';
    const RESOURCE_LABEL =          'Colaborador';
    CONST RESOURCE_CONTROL =        'R';
    const CUSTOMER_LABEL =          'Cliente';
    CONST CUSTOMER_CONTROL =        'C';

    public function __construct(EntityManager $entityManager, \Util\Utilities $environment ,\Factory\Profile $profileFactory){
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->profileFactory = $profileFactory;
    }

    public function getProfileByName($category){
		/** @var \Zeedhi\DevOps\Model\Projectmanagement\Repositories\Profile $profileRepository */
		$profileRepository = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Profile');
		return $profileRepository->getProfileByName($category, $this->environment->getCurrentOrganizationId());
    }

    public function createProfile($category, $label, $status, $structure, $control = null) {
        $entity = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Profile')
            ->findOneBy(array("label" => $label, "organization" => $this->environment->getCurrentOrganizationId()));

        if(empty($entity)) {
            $entity = $this->profileFactory->createProfile($category, $label, $status, $structure, $control);
            $this->entityManager->persist($entity);
            $this->entityManager->flush();
        }
        else {
            throw new \Exception('já existe um perfil de acesso com esse rótulo nessa mesma organização. ', 1);
        }

        return $entity;
    }

    public function updateProfile($profile, $category, $label, $status, $structure, $control, $businessPartner) {
        $entity = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Profile')
            ->findOneBy(array("label" => $label, "organization" => $this->environment->getCurrentOrganizationId()));

        if(empty($entity)) {
            $entity = $this->categoryFactory->updateProfile($profile, $category, $label, $status, $structure, $control, $businessPartner);
            $this->entityManager->persist($entity);
            $this->entityManager->flush();
        }
        else if($profile->getId() !== $entity->getId()){
            throw new \Exception('já existe um perfil de acesso com esse rótulo nessa mesma organização.', 1);
        }

        return $entity;
    }
}