<?php
namespace Util;

/**
 * Class Utilities
 *
 * @package Util
 */

use Doctrine\ORM\EntityManager;
use Util\Events\Audit\DevOpsEnvironment;
use Zeedhi\DevOps\Model\Profile\Entities\ZhUser;
use Zeedhi\DevOps\Util\Events\Audit_with_id\Environment as Audit;
use Zeedhi\DevOps\Util\Events\Organization_with_id\Environment as Organization;
use Zeedhi\Framework\DependencyInjection\InstanceManager;
use Zeedhi\Framework\ORM\DateTime;
use Zeedhi\Framework\Session\Session;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\DevOps\Model\Projectmanagement\Entities;

class Utilities implements Audit, Organization
{
    public $PRIVATE_KEY_DEVOPS  = "do25449996c21f780fe30badd6521931";
    public $CLIENT_ID  = "cd7e82153be60953002055f9b19a50a4";
    public $DEVOPS_URL;
    public $zmartUrl;
    public $idViewZmartDashboard;
    public $zmartStructureFilterId;
    public $addDayInDate;

    /** @var \DateTime(); */
    protected $currentTime;
    /** @var Session */
    protected $session;
    const ACTIVE = 'A';
    const INACTIVE = 'I';
    const FORMAT_DATE_DEFAULT_NO_SECS = 'd/m/Y H:i';
    const FORMAT_DATE_DEFAULT = 'd/m/Y H:i:s';
    const FORMAT_DATE_DEFAULT_ONLY_DATE = 'd/m/Y';
    const FORMAT_DATE_DEFAULT_ONLY_DATE_AMERICAN = 'Y-m-d';
    const FORMAT_DATE_DEFAULT_BY_BD = 'Y-m-d H:i:s';
    const REPOSITORY_DIR = '\Zeedhi\DevOps\Model\Projectmanagement\Entities\\';
    const ENTITY_DIR = '\Zeedhi\DevOps\Model\Projectmanagement\Entities\\';
    const PROFILE_ENTITY_DIR = '\Zeedhi\DevOps\Model\Profile\Entities\\';
    const CONFIGURATION_ENTITY_DIR = '\Zeedhi\DevOps\Model\Configurationmanager\Entities\\';
    const SERVER_ENTITY_DIR = '\Zeedhi\DevOps\Model\Servermanager\Entities\\';
    const GERAL_ENTITY_DIR = '\Teknisa\Model\Geral\Entities\\';
    const TEKNISA_ENTITY_DIR = '\Teknisa\Model\Teknisa\Entities\\';
    const IS_ACTIVE_YES = 'Y';
    const IS_ACTIVE_NO  = 'N';
    const CLIENT_ID_LIBRARIES = 'cd7e85178153by60953518945f9hakt19a50a4';
    const CLIENT_SECRET_LIBRARIES = 'lib5449996c21f780fe30badd6521931a5sac9';

    public static $ARRAY_MONTH_FOR_NUMERIC = array(
        'Janeiro' => 1,
        'Fevereiro' => 2,
        'Março' => 3,
        'Abril' => 4,
        'Maio' => 5,
        'Junho' => 6,
        'Julho' => 7,
        'Agosto' => 8,
        'Setembro' => 9,
        'Outubro' => 10,
        'Novembro' => 11,
        'Dezembro' => 12,
    );
    public static $ARRAY_NUMERIC_FOR_MONTH = array(
        NULL,
        'Janeiro',
        'Fevereiro',
        'Março',
        'Abril',
        'Maio',
        'Junho',
        'Julho',
        'Agosto',
        'Setembro',
        'Outubro',
        'Novembro',
        'Dezembro'
    );
    public static $ARRAY_WEEK_BY_BD = array(
        NULL, 'SUN', 'MON', 'TUE', 'WED', 'THU', 'FRI', 'SAT'
    );
    public static $ARRAY_WEEK_NAME_BY_BD = array(
        NULL, 'SUNDAY', 'MONDAY', 'TUESDAY', 'WEDNESDAY', 'THURSDAY', 'FRIDAY', 'SATURDAY'
    );

    /**
     * @var Messages
     */
    public $messages;
    /**
     * @var DevOpsEnvironment
     */
    public $devOpsEnvironment;

    public function __construct(Session $session, \Util\Messages $messages, DevOpsEnvironment $devOpsEnvironment, $devOpsUrl, $zmartUrl, $idViewZmartDashboard, $zmartStructureFilterId, $addDayInDate)
    {
        $this->session = $session;
        $this->currentTime = new \DateTime();
        $this->messages = $messages;
        $this->devOpsEnvironment = $devOpsEnvironment;
        $this->DEVOPS_URL = $devOpsUrl;
        $this->zmartUrl = $zmartUrl;
        $this->zmartStructureFilterId = $zmartStructureFilterId;
        $this->idViewZmartDashboard = $idViewZmartDashboard;
        $this->addDayInDate = $addDayInDate;
    }

    public static function parseStringToDate($stringDate)
    {
        if (!empty($stringDate)) {
            $date = new \DateTime($stringDate);
        } else {
            $date = null;
        }
        return $date;
    }

    public static function validMd5($string)
    {
        return preg_match('/^[a-f0-9]{32}$/', $string);
    }
    public static function md5CodeIfNecessary($string)
    {
        $is_valid_md5 = self::validMd5($string);
        if ($is_valid_md5) {
            $md5String = $string;
        } else {
            $md5String = md5($string);
        }
        return $md5String;
    }

    /**
     * @param array[] $filters
     * @param $nameFilter
     * @param bool $remove
     * @return \DateTime|null
     */
    public static function getValueInFilterDataSourceByName(array &$filters,$nameFilter,$remove = false)
    {
        foreach ($filters as $key => $filter) {
            $name = $filter['columnName'];
            $values = $filter['value'];
            if($name == $nameFilter ){
                if($remove){
                    unset($filters[$key]);
                }
                return $values;
            }
        }
        return null;
    }

    public static function parseTimeToSeconds($time)
    {
        if (!empty($time)) {
            $hour = explode(':', $time);
            $minutes = $hour[0] * 60 + $hour[1];
            $seconds = $minutes * 60;
        } else {
            $seconds = 0;
        }
        return $seconds;
    }

    public function getStructureForResource(\Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource $resource){
        $entityManager = self::getEntityManager();
        /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\Permission[] $permissions */
        $permissions = $entityManager->getRepository(Utilities::ENTITY_DIR . 'PERMISSION')
            ->findBy(array('resource' =>$resource->getId()));
        foreach($permissions as $permission){
            if(in_array($permission->getType(),array(null,\Service\Permission::IS_RESOURCE))){
                return $permission->getStructure();
            }
        }
        return null;
    }

    public static function moneyForDouble($real)
    {
        $symbolMoney = array('.', ',', 'R$');
        $symbolDouble = array('', '.', '');
        $value = str_replace($symbolMoney, $symbolDouble, $real);
        return (double)$value;
    }

    public static function setSelectMultiInQuery($stringValue, $int = false, $delimiter = ',')
    {
        if (empty($stringValue)) {
            $value = array();
        } else {
            if ($int) {
                $value = array_map('intval', explode($delimiter, $stringValue));
            } else {
                $value = explode($delimiter, $stringValue);
            }
        }
        return array_values($value);
    }

    public function getCurrentTime()
    {
        return clone $this->currentTime;
    }

    public static function getDateNowString()
    {
        $date = new \DateTime();
        return $date->format(Utilities::FORMAT_DATE_DEFAULT);
    }
    
    public function getCurrentUserId()
    {
        return $this->session->get('userId');
    }

    public function getCurrentUserName()
    {
        return $this->session->get('userName');
    }

    public function setCurrentUserName($name)
    {
        $this->session->set('userName',$name);
    }

    public function getCurrentUserEmail()
    {
        return $this->session->get('userEmail');
    }

    public function setCurrentUserEmail($name)
    {
        $this->session->set('userEmail',$name);
    }

    public function setCurrentOrganizationId($name)
    {
        $this->session->set('organizationId',$name);
    }

    public function getCurrentOrganizationName()
    {
        return $this->session->get('organizationName');
    }

    public function setCurrentOrganizationName($name)
    {
        $this->session->set('organizationName',$name);
    }
    public function getCurrentResourceId()
    {
        return $this->session->get('resourceId');
    }

    public function setCurrentResourceId($resourceId)
    {
        $this->session->set('resourceId',$resourceId);
    }

    public function getCurrentCdoperador()
    {
        return $this->session->get('USER_ID');
    }
    public function setCurrentCdoperador($cdoperador)
    {
        return $this->session->set('USER_ID',$cdoperador);
    }

    public function getDOCurrentUserId()
    {
        return $this->devOpsEnvironment->getCurrentUserId();
    }

    public function getCurrentOrganizationId()
    {
        return $this->session->get('organizationId');
    }

    public function getCurrentExternalOrganizationId()
    {
        return $this->session->get('externalOrganizationId');
    }

    public function setCurrentExternalOrganizationId($externalOrgId)
    {
        $this->session->set('externalOrganizationId', $externalOrgId);
    }

    public function getOrganizationId() {
        return $this->session->has("organizationId") ? $this->session->get("organizationId") : null;
    }
    public function setCurrentUserProfiles($externalOrgId)
    {
        $this->session->set('profileIds', $externalOrgId);
    }

    public function getCurrentUserProfiles() {
        return $this->session->has("profileIds") ? $this->session->get("profileIds") : null;
    }

    public function getProjectId() {
        return 4;
    }

    public function getProjectUniqueId() {
        return "PM_4";
    }

    public function getToken () {
        return $this->session->get("token");
    }

    public function setToken ($token) {
        $this->session->set("token", $token);
    }

    public function getTokenDevOps () {
        return $this->session->get("tokenDevOps") ? $this->session->get("tokenDevOps") : null;
    }

    public function setTokenDevOps ($token) {
        $this->session->set("tokenDevOps", $token);
    }

    public function getAccessProfile(){
        $this->session->get("ACCESS_PROFILE");
    }
    
    public static function getNextCalendar(\Zeedhi\DevOps\Model\Projectmanagement\EntitiesInterface\Calendar\EntityCalendarUserInterface $entity){
        if(!$entity->getCalendar() && $entity->getParent()){
            $calendar = self::getNextCalendar($entity->getParent());
        }else{
            $calendar = $entity->getCalendar();
        }
        return $calendar;
    }

    /**
     * @param \Zeedhi\DevOps\Model\Projectmanagement\EntitiesInterface\Calendar\EntityCalendarUserInterface $entity
     * @return \Zeedhi\DevOps\Model\Projectmanagement\EntitiesInterface\Calendar\EntityCalendarUserInterface
     */
    public static function getNextEntityWithCalendar(\Zeedhi\DevOps\Model\Projectmanagement\EntitiesInterface\Calendar\EntityCalendarUserInterface $entity){
        if(!$entity->getCalendar() && $entity->getParent()){
            $entityCalendar = self::getNextEntityWithCalendar($entity->getParent());
        }else{
            $entityCalendar = $entity;
        }
        return $entityCalendar;
    }
    public function getCurrentOrganization()
    {
        $entityManager = self::getEntityManager();
        return $entityManager->find('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Organization', $this->getCurrentOrganizationId());
    }

    public static function getTableName($entity)
    {
        $entityManager = self::getEntityManager();
        $entityMetadata = $entityManager->getClassMetadata(get_class($entity));
        $tableName = $entityMetadata->getTableName();
        return $tableName;
    }

    public static function getCodeByTableName($tableName,$nrorg){
        return self::getNewCode()->getCodeByTableName($tableName,$nrorg);
    }
    public static function getAlphanumericCodeByTableName($tableName,$nrorg){
        return self::getAlphanumericNewCode()->getCodeByTableName($tableName,$nrorg);
    }
    /** @return \Service\Teknisa\NewCode */
    public static function getNewCode(){
        return InstanceManager::getInstance()->getService('\Service\Teknisa\NewCode');
    }
    /**
     * @return \Service\Teknisa\AlphanumericNewCode
     */
    public static function getAlphanumericNewCode(){
        return InstanceManager::getInstance()->getService('\Service\Teknisa\AlphanumericNewCode');
    }

    /**
     * @return \Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource
     */
    public function getCurrentUser()
    {
        $entityManager = self::getEntityManager();
        return $entityManager->find(self::ENTITY_DIR . 'Resource', $this->getCurrentUserId());
    }
    /**
     * @return ZhUser
     */

    public function getDOCurrentUser()
    {
        $entityManager = self::getEntityManager();
        return $entityManager->find(self::PROFILE_ENTITY_DIR . 'ZhUser', $this->getDOCurrentUserId());
    }

    public function setCurrentUserId($userId)
    {
        $this->session->set('userId', $userId);
        $this->setDOCurrentUserIdByCurrentPMUser();
    }

    public function setOrganizationId($orgId)
    {
        $this->session->set('organizationId', $orgId);
    }

    /**
     * @param Entities\Resource[] $resources
     * @return ZhUser[]
     */
    public static function getZhUserByResource(array $resources)
    {
        $externalIds = self::getArrayByArrayObjectWithGet($resources);
        $entityManager = self::getEntityManager();
        return $entityManager->getRepository(self::PROFILE_ENTITY_DIR . 'ZhUser')
            ->findBy(array('externalResId'=>$externalIds));
    }

    /**
     * @param String $email
     * @return int
     */
    public static function validateEmail($email)
    {
        $emailRegex = "/^[a-zA-Z0-9.!#$%&'*+\/=?^_`{|}~-]+@[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?(?:\.[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?)*$/";
        return preg_match($emailRegex, $email);
    }

    public static function validateNameLogin($name)
    {
        $nameRegex = "/^[a-zA-ZàáâäãåąčćęèéêëėįìíîïłńòóôöõøùúûüųūÿýżźñçčšžÀÁÂÄÃÅĄĆČĖĘÈÉÊËÌÍÎÏĮŁŃÒÓÔÖÕØÙÚÛÜŲŪŸÝŻŹÑßÇŒÆČŠŽ∂ð ,.'-]+$/";
        return preg_match($nameRegex, $name);
    }

    public static function validateDocumentNumber($documentNumber)
    {
        if ($documentNumber) {
            return preg_replace("/[^0-9]/", "", $documentNumber);
        }
        return $documentNumber;
    }

    public static function secondsToTime($seconds,$withSeconds = true)
    {
        $time = '';
        $hours = $seconds / 3600;
        if ($hours >= 1) {
            $time .= floor($hours) . 'H';
        }
        $minutes = ($hours - floor($hours)) * 60;
        $seconds = ($minutes - floor($minutes)) * 60;
        $seconds = round($seconds);
        if ($seconds == 60) {
            $minutes += 1;
            $seconds = 0;
        }
        if ($minutes >= 1) {
            $time .= floor($minutes) . 'm';
        }
        if ($withSeconds && $seconds >= 1) {
            $time .= $seconds . 's';
        }
        return $time;
    }

    public static function timeToSeconds($time,$withSeconds = true)
    {   $time = strtoupper($time);
        $hourSeparator = strpos($time, 'H');
        $seconds = 0;
        if ($hourSeparator) {
            $seconds = intval(substr($time, 0, $hourSeparator)) * 3600;
        } else {
            $hourSeparator = -1;
        }
        $minuteSeparator = strpos($time, 'M');
        if ($minuteSeparator) {
            $seconds += intval(substr($time, $hourSeparator + 1, $minuteSeparator)) * 60;
        } else {
            $minuteSeparator = -1;
        }
        $secondSeparator = strpos($time, 'S');
        if ($withSeconds && $secondSeparator) {
            $seconds += intval(substr($time, $minuteSeparator + 1, $secondSeparator));
        }
        return $seconds;
    }

    public static function executeFunction($object,$arrayMethod){
        foreach ($arrayMethod as $method) {
            $object = $object->$method();
        }
        return $object;
    }
    public static function getArrayByArrayObjectWithGet( array $arrayObject = array(),$arrayMethod = array('getId')){
        $array = array();
        foreach ($arrayObject as $object) {
            $array[] = self::executeFunction($object,$arrayMethod);
        }
        return $array;
    }
    /**
     * @param $entity
     * @return mixed
     */
    public static function getLastNameClass($entity)
    {
        $directoryClassSeparator = str_replace("'", "", "\'");
        $className2 = explode($directoryClassSeparator, get_class($entity));
        $className = array_pop($className2);
        return $className;
    }

    /**
     * When you use delete from CRUD, if the delete operation
     * gets an error, the error is not correctly treated and
     * a popup with error does not appear for final user.
     * This method fix it, but it may have problems on future.
     * @param Response $response A \DTO\Response
     * @param string $errorMessage A message that will be displayed to user
     */
    public static function treatResponseError(Response $response, $errorMessage)
    {
        if ($error = $response->getError()) {
            if (preg_match('/integridade/', $error->getMessage())
                || preg_match('/integrity/', $error->getMessage())
            ) {
                $response->addMessage(new Response\Message($errorMessage));
                $response->setStatus(Response::STATUS_SUCCESS);
            }
        }
    }

    /**
     * Tree Grid has a problem: when he gets a data that has
     * an item with PARENT equals a value that does not exists
     * in Database, it just disappears.
     * So, this method is a little tool, setting non-existent
     * IDs as null, avoiding problem with Tree Grid.
     * @param array $values
     * @param string $indexPrimary
     * @param string $indexParent
     * @return mixed
     */
    public static function fixParentsForTreeGrid(array $values,$indexPrimary = 'ID',$indexParent = 'PARENT')
    {
        $aux = array();
        foreach ($values as $item) {
            array_push($aux, $item[$indexPrimary]);
        }
        foreach ($values as &$item) {
            if (!in_array($item[$indexParent], $aux)) {
                $item[$indexParent] = null;
            }
        }
        return $values;
    }

    /**
     * When you use CRUD for find(), Zeedhi attrs some
     * values to returned dataSet, as __is_new, for example.
     * So, when you are not using default methods from CRUD,
     * can use this snippet for fix it.
     * @param $dataSet
     * @return mixed
     */
    public static function setEssentialValuesToDataset($dataSet)
    {
        foreach ($dataSet as &$eachDataSet) {
            $eachDataSet['__is_new'] = false;
        }
        return $dataSet;
    }

    public static function getDateNowDB()
    {
        /** @var EntityManager $entityManager */
        $entityManager = self::getEntityManager();
        $result = $entityManager->getConnection()->fetchAll('SELECT SYSDATE FROM DUAL');
        return new \DateTime($result[0]['SYSDATE']);
    }

    public static function compareTwoDates(\DateTime $date1, \DateTime $date2)
    {
        return $date1->format('Ymd') == $date2->format('Ymd');
    }

    public static function isDateToday(\DateTime $date)
    {
        return self::compareTwoDates($date, new \DateTime());
    }


    public static function removeArrayEntities(EntityManager $entityManager, $arrayEntities = array()){
        foreach ($arrayEntities as $entity) {
            $entityManager->remove($entity);
        }
    }

    /**
     * When you filters a datasource (like filter actions),
     * this method automates somethings for you.
     * @param $filters
     * @param $tableAlias
     * @return string
     */
    //TODO @tuliomagalhaes, @rafaelwemerson, @stanleypinto, @phillipesilva - Precisa muito melhorar isso aqui!
    public static function createQueryFiltered($filters, $tableAlias = null)
    {
        $filterSQL = '';
        foreach ($filters as $filter) {
            $name = $filter['columnName'];
            $operator = $filter['operator'] == "LIKE_I" ? "LIKE" : $filter['operator'];
            $values = $filter['value'];
            if (($name != '*' && !empty($values)) || $values == 0) {
                if ($operator == 'BETWEEN') {
                    if ($values[0] && $values[1]) {
                        $values = "TO_DATE( '$values[0] 00:00:00' ,'DD/MM/YYYY HH24:MI:SS')
                                AND TO_DATE( '$values[1] 23:59:59' ,'DD/MM/YYYY HH24:MI:SS')";
                    } else if ($values[0]) {
                        $operator = '>';
                        $values = "TO_DATE( '$values[0] 00:00:00' ,'DD/MM/YYYY HH24:MI:SS')";
                    } else if ($values[1]) {
                        $operator = '<';
                        $values = "TO_DATE( '$values[1] 00:00:00' ,'DD/MM/YYYY HH24:MI:SS')";
                    } else {
                        $values = "";
                    }

                    if ($values || ($values && ($values[0] || $values[1]))) {
                        if (!empty($tableAlias)) {
                            $filterSQL .= "AND $tableAlias.$name $operator $values";
                        } else {
                            $filterSQL .= "AND $name $operator $values";
                        }
                    }
                } else {
                    if (is_array($values) && $operator != 'MAPPED_LIKE_ALL') {
                        $stringValues = "('" . implode($values, "','") . "')";
                        $values = $stringValues;
                    }
                    if ($operator == 'LIKE_ALL') {
                        if (!empty($tableAlias)) {
                            $values = "UPPER('" . $values . "')";
                        } else {
                            $values = "UPPER('" . $values . "')";
                        }

                        $columnList = explode('|', $name);
                        $likes = array();
                        foreach ($columnList as $column) {
                            $likes[] = "UPPER($column) LIKE $values";
                        }
                        $filterLikeAll = implode($likes, ' OR ');
                        $filterSQL .= "AND ( $filterLikeAll )";

                    } else if ($operator == 'LIKE') {
                        if (!empty($tableAlias)) {
                            $values = "UPPER('" . $values . "')";
                            $name = "UPPER($tableAlias.$name)";
                        } else {
                            $values = "UPPER('" . $values . "')";
                            $name = "UPPER($name)";
                        }
                        $filterSQL .= "AND $name $operator $values";
                    } else if ($operator == 'MAPPED_LIKE_ALL') {
                        foreach($values[0] as $index => $mappedLikeAllValues) {
                            if($index == '_ALL') {
                                if (!empty($tableAlias)) {
                                    $values = "UPPER('" . $mappedLikeAllValues . "')";
                                } else {
                                    $values = "UPPER('" . $mappedLikeAllValues . "')";
                                }

                                $columnList = explode('|', explode('#', $name)[0]);
                                $likes = array();
                                foreach ($columnList as $column) {
                                    $likes[] = "UPPER($column) LIKE $values";
                                }
                                $filterLikeAll = implode($likes, ' OR ');
                                $filterSQL .= " AND ( $filterLikeAll )";
                            } else {
                                $stringValues = "('" . implode($mappedLikeAllValues, "','") . "')";
                                if (!empty($tableAlias)) {
                                    $filterSQL .= " AND $tableAlias.$index IN $stringValues ";
                                } else {
                                    $filterSQL .= " AND $index IN $stringValues ";
                                }
                            }
                        }
                    } else {
                        if (!empty($tableAlias)) {
                            $filterSQL .= "AND $tableAlias.$name $operator $values";
                        } else {
                            $filterSQL .= "AND $name $operator $values";
                        }
                    }
                }
            }
        }
        return $filterSQL;
    }
    
    public static function getFilterTableColumn($filters, $tableColumn)
    {
        $filterLabel = [];
        foreach ($filters as $filter) {
            if ($filter['columnName'] === $tableColumn) {
               $filterLabel =  [$filter];
            }
        }
        return $filterLabel;
    }


    public static function getAllStructureLessId($structureIds)
    {
        $structureString = '';
        foreach ($structureIds as $structureId) {
            $structureString .= (string)$structureId['ID'].',';
        }
        $structureString .= '0';
        $query = <<<SQL
                SELECT P.ID
                FROM STRUCTURE P
                WHERE P.ID IN ($structureString)
                AND P.STATUS <> 'D'
                UNION ALL
                SELECT P.ID
                FROM STRUCTURE P
                WHERE P.STATUS <> 'D'
                START WITH P.PARENT IN ($structureString)
                CONNECT BY PRIOR P.ID = P.PARENT
SQL;
        return self::getEntityManager()->getConnection()->fetchAll($query);
    }


    /**
     * Gets dir adding .git extension.
     * @param $dir
     * @return mixed
     */
    public function getRootDir($dir)
    {
        if (file_exists($dir . DIRECTORY_SEPARATOR . 'mobile')) {
            return $dir;
        } else {
            return $this->getRootDir(dirname($dir));
        }
    }

    public static function getDayWeekPortugueseByNumber($weekNumber)
    {
        $dayWeek = \Service\Calendar\TaskCalendar::$ARRAY_WEEK_BR_LABEL[intval($weekNumber) - 1];
        return $dayWeek;

    }


    /************************************************************
     * Gets a month name by his "year index" (1-12).
     * @param    int $monthNumber
     * @param    bool $upperCased
     * @return    string    $monthNames[index]
     ************************************************************/
    public function getMonthNameByMonthNumber($monthNumber, $upperCased = 'none', $abbreviated = false){
        $monthNumber -= 1; // decrements for work with array
        $monthName = 'janeiro';
        $monthNames = array(
            'janeiro',
            'fevereiro',
            'março',
            'abril',
            'maio',
            'junho',
            'julho',
            'agosto',
            'setembro',
            'outubro',
            'novembro',
            'dezembro'
        );
        if ($upperCased == 'all') {
            $monthName = strtoupper($monthNames[$monthNumber]);
        } elseif ($upperCased == 'first') {
            $monthName = ucfirst($monthNames[$monthNumber]);
        } elseif ($upperCased == 'none' || $upperCased == false) {
            $monthName = $monthNames[$monthNumber];
        } else {
            throw new \Exception("getMonthNameByMonthNumber() expected all|first|none string, got {$upperCased} " . gettype($upperCased));
        }

        if ($abbreviated) {
            $monthName = substr($monthName, 0, 3);
        }

        return $monthName;
    }

    /****************************************************************************
     * Gets current date written in full format (brazilian format)
     ****************************************************************************/
    public function getDateWrittenInFullByDateFormat($firstSeparator = '/', $secondSeparator = '/'){
        $date       = $this->getCurrentTime();
        $day        = $date->format('d');
        $month      = $date->format('m');
        $year       = $date->format('Y');
        $monthName  = $this->getMonthNameByMonthNumber($month);
        return "{$day}{$firstSeparator}{$monthName}{$secondSeparator}{$year}";
    }

    public static function getUniqueArrayMultidimensional($array){
        $returnArray = array();
        $array = array_map("unserialize", array_unique(array_map("serialize", $array)));
        foreach ($array as $value){
            $returnArray[] = $value;
        }
        return  $returnArray;
    }

    public static function getValueIndexInArray(array $array,$index){
        $returnArray = array();
        foreach ($array as $value){
            $returnArray[] = $value[$index];
        }
        return  $returnArray;
    }

    /**
     * @param $month
     * @param $year
     * @return array
     */
    public static function getBeginAndEndDateByMonth($month, $year)
    {
        if (empty($month) || empty($year)) {
            $beginDate = new \DateTime('now');
            $endDate = new \DateTime('now');
        } else {
            $beginDate = \DateTime::createFromFormat('m Y', $month.' '.$year);
            $endDate = \DateTime::createFromFormat('m Y', $month.' '.$year);
        }
        $beginDate = $beginDate->modify('first day of this month')->modify('-7 day')->format(Utilities::FORMAT_DATE_DEFAULT_ONLY_DATE);
        $endDate = $endDate->modify('last day of this month')->modify('+14 day')->format(Utilities::FORMAT_DATE_DEFAULT_ONLY_DATE);
        return array($beginDate, $endDate);
    }
    
    /**
     * @return array
     */
    public static function getBeginAndEndDateByCurrentMonthInArray()
    {
        $currentDate = new \DateTime('now');
        $beginDate = $currentDate->modify('first day of this month')->format(Utilities::FORMAT_DATE_DEFAULT_ONLY_DATE);
        $endDate = $currentDate->modify('last day of this month')->format(Utilities::FORMAT_DATE_DEFAULT_ONLY_DATE);
        return array($beginDate, $endDate);
    }

    /**
     * @param $calendarDates
     * @param $date
     * @return bool
     * @throws \Exception
     */
    public static function dateNotUtil($calendarDates, $date)
    {
        foreach($calendarDates as $calendarDate){
            if($calendarDate['DATA'] == $date ){
                if( $calendarDate['TYPE'] == \Service\CalendarDetail::OUT_DAY_TYPE){
                    return true;
                }else{
                    return false;
                }
            }
        }
        throw new \Exception('date not found in CalendarDates');
        return false;
    }

    public function setDOCurrentUserIdByCurrentPMUser()
    {
        /** @var EntityManager $entityManager */
        $entityManager = self::getEntityManager();
        $PMCurrentUser = $this->getCurrentUser();
        /** @var  ZhUser $DOCurrentUser */
        $DOCurrentUser = $entityManager->getRepository(self::PROFILE_ENTITY_DIR . 'ZhUser')
            ->findOneBy(array('externalId'=>$PMCurrentUser->getExternalId()));
        
        if($DOCurrentUser){
            $this->devOpsEnvironment->setCurrentUserId($DOCurrentUser->getId());
        }else{
            $DOCurrentUser = $entityManager->getRepository(self::PROFILE_ENTITY_DIR . 'ZhUser')
                ->findOneBy(array('externalResId'=>$PMCurrentUser->getId()));
            if($DOCurrentUser){
                $this->devOpsEnvironment->setCurrentUserId($DOCurrentUser->getId());
            }else{
                $this->devOpsEnvironment->setCurrentUserId(1);
            }
        }
    }

    //tasks status
    /**
     * @return Entities\TaskStatus
     */
    public static function getTaskStatusNotInitialized(){
        /** @var EntityManager $entityManager */
        $entityManager = self::getEntityManager();
        return $entityManager->getRepository(self::ENTITY_DIR . 'TaskStatus')
            ->findOneBy(array('control'=>Entities\TaskStatus::TASK_STATUS_CONTROL_NOT_INITIALIZED, 'isProject' => 'N' ));
    }
    /**
     * @return Entities\TaskStatus
     */
    public static function getTaskStatusBacklog(){
        /** @var EntityManager $entityManager */
        $entityManager = self::getEntityManager();
        return $entityManager->getRepository(self::ENTITY_DIR . 'TaskStatus')
            ->findOneBy(array('control'=>Entities\TaskStatus::TASK_STATUS_CONTROL_BACKLOG, 'isProject' => 'N' ));
    }
    /**
     * @return Entities\TaskStatus
     */
    public static function getTaskStatusIsDeveloping(){
        /** @var EntityManager $entityManager */
        $entityManager = self::getEntityManager();
        return $entityManager->getRepository(self::ENTITY_DIR . 'TaskStatus')
            ->findOneBy(array('control'=>Entities\TaskStatus::TASK_STATUS_CONTROL_IN_DEVELOPING, 'isProject' => 'N' ));
    }
    /**
     * @return Entities\TaskStatus
     */
    public static function getTaskStatusPaused(){
        /** @var EntityManager $entityManager */
        $entityManager = self::getEntityManager();
        return $entityManager->getRepository(self::ENTITY_DIR . 'TaskStatus')
            ->findOneBy(array('control'=>Entities\TaskStatus::TASK_STATUS_CONTROL_PAUSED, 'isProject' => 'N' ));
    }
    /**
     * @return Entities\TaskStatus
     */
    public static function getTaskStatusIsConcluded(){
        /** @var EntityManager $entityManager */
        $entityManager = self::getEntityManager();
        return $entityManager->getRepository(self::ENTITY_DIR . 'TaskStatus')
            ->findOneBy(array('control'=>Entities\TaskStatus::TASK_STATUS_CONTROL_CONCLUDED, 'isProject' => 'N' ));
    }
    /**
     * @return Entities\TaskStatus
     */
    public static function getTaskStatusHomologated(){
        /** @var EntityManager $entityManager */
        $entityManager = self::getEntityManager();
        return $entityManager->getRepository(self::ENTITY_DIR . 'TaskStatus')
            ->findOneBy(array('control'=>Entities\TaskStatus::TASK_STATUS_CONTROL_HOMOLOGATED, 'isProject' => 'N' ));
    }

    //projects status
    /**
     * @return Entities\TaskStatus
     */
    public static function getProjectStatusPlanning(){
        /** @var EntityManager $entityManager */
        $entityManager = self::getEntityManager();
        return $entityManager->getRepository(self::ENTITY_DIR . 'TaskStatus')
            ->findOneBy(array('control'=>Entities\TaskStatus::TASK_STATUS_CONTROL_PLANNING, 'isProject' => 'Y' ));
    }
    /**
     * @return Entities\TaskStatus
     */
    public static function getProjectStatusProgress(){
        /** @var EntityManager $entityManager */
        $entityManager = self::getEntityManager();
        return $entityManager->getRepository(self::ENTITY_DIR . 'TaskStatus')
            ->findOneBy(array('control'=>Entities\TaskStatus::TASK_STATUS_CONTROL_PROGRESS, 'isProject' => 'Y' ));
    }
    /**
     * @return Entities\TaskStatus
     */
    public static function getProjectStatusCanceled(){
        /** @var EntityManager $entityManager */
        $entityManager = self::getEntityManager();
        return $entityManager->getRepository(self::ENTITY_DIR . 'TaskStatus')
            ->findOneBy(array('control'=>Entities\TaskStatus::TASK_STATUS_CONTROL_CANCELED, 'isProject' => 'Y' ));
    }
    /**
     * @return Entities\TaskStatus
     */
    public static function getProjectStatusNumb(){
        /** @var EntityManager $entityManager */
        $entityManager = self::getEntityManager();
        return $entityManager->getRepository(self::ENTITY_DIR . 'TaskStatus')
            ->findOneBy(array('control'=>Entities\TaskStatus::TASK_STATUS_CONTROL_NUMB, 'isProject' => 'Y' ));
    }
    /**
     * @return Entities\TaskStatus
     */
    public static function getProjectStatusConcluded(){
        /** @var EntityManager $entityManager */
        $entityManager = self::getEntityManager();
        return $entityManager->getRepository(self::ENTITY_DIR . 'TaskStatus')
            ->findOneBy(array('control'=>Entities\TaskStatus::TASK_STATUS_CONTROL_CONCLUDED, 'isProject' => 'Y' ));
    }

    /**
     * @return Entities\ProjectType
     */

    public static function getProjectTypeMeeting(){
        /** @var EntityManager $entityManager */
        $entityManager = self::getEntityManager();
        return $entityManager->getRepository(self::ENTITY_DIR . 'ProjectType')
            ->findOneBy(array('externaltype'=>Entities\ProjectType::TYPE_PROJECT_CONTROL_MEETING));
    }

    public static function getProjectTypeTimeLine(){
        /** @var EntityManager $entityManager */
        $entityManager = self::getEntityManager();
        return $entityManager->getRepository(self::ENTITY_DIR . 'ProjectType')
            ->findOneBy(array('externaltype'=>Entities\ProjectType::TYPE_PROJECT_CONTROL_TIME_LINE));
    }
    /**
     * @return Entities\ProjectType
     */

    public static function getProjectTypeDefault(){
        /** @var EntityManager $entityManager */
        $entityManager = self::getEntityManager();
        return $entityManager->getRepository(self::ENTITY_DIR . 'ProjectType')
            ->findOneBy(array('externaltype'=>Entities\ProjectType::TYPE_PROJECT_CONTROL_STANDARD));
    }

    /**
     * @return Entities\ProjectType
     */
    public static function getProjectTypeMeetingOccurrences(){
        /** @var EntityManager $entityManager */
        $entityManager = self::getEntityManager();
        return $entityManager->getRepository(self::ENTITY_DIR . 'ProjectType')
            ->findOneBy(array('externaltype'=>Entities\ProjectType::TYPE_PROJECT_CONTROL_MEETING_OCCURRENCES));
    }
    /** @return \Doctrine\ORM\EntityManager */
    public static function getEntityManager(){
        return InstanceManager::getInstance()->getService('entityManager');
    }

    /**
     * @return Utilities
     */
    public static function getEnvironment(){
        return InstanceManager::getInstance()->getService('\Util\Utilities');
    }

    public static function getCurrentUserIdPm(){
        return InstanceManager::getInstance()->getParameter('CURRENT_USER_ID_PM');
    }
    public static function getDirectoryBasicUpload(){
        return InstanceManager::getInstance()->getParameter('ATTACHMENTS_BASE_DIRECTORY').DIRECTORY_SEPARATOR.'attachments';
    }

    public static function getCurrentOrganizationIdPm(){
        return InstanceManager::getInstance()->getParameter('CURRENT_ORGANIZATION_ID_PM');
    }

    /**
     * @return \Service\Project
     */
    public static function getProjectService(){
        return InstanceManager::getInstance()->getService('\Service\Project');
    }

    /**
     * @return \Service\Task
     */
    public static function getTaskService(){
        return InstanceManager::getInstance()->getService('\Service\Task');
    }
    /**
     * @return \Service\Comment
     */
    public static function getCommentService(){
        return InstanceManager::getInstance()->getService('\Service\Comment');
    }
    /**
     * @return \Service\Attachment
     */
    public static function getAttachmentService(){
        return InstanceManager::getInstance()->getService('\Service\Attachment');
    }

    public static function compareDates($a, $b){
        $return = intval(\DateTime::createFromFormat('d/m/Y', $a)
            ->diff(\DateTime::createFromFormat('d/m/Y', $b))->format('%r%a'));
        return $return === 0 ? 0 : $return > 0 ? 1 : -1;
    }

    /**
     * @param $array
     * @param String $key
     * @return array
     */
    public static function getArrayAssociativeFromArray( array $array = array(), $key = 'ID')
    {
        $result = array();
        foreach ($array as $row) {
            $result[$row[$key]] = $row;
        }
        return $result;
    }
    
    public static function getArrayFromAssociativeArray( array $array = array(), $key = 'ID')
    {
        $result = array();
        foreach ($array as $row) {
            $result[] = $row[$key] ;
        }
        return $result;
    }

    public static function callWebService($token, $requestType, $data, $dataSetResultPos, $privateKey, $url)
    {
        set_time_limit(180);
        $header = array();
        $header[] = 'OAuth-Token: ' . $token;
        $header[] = 'OAuth-SecretId: ' . $privateKey;
        $header[] = 'OAuth-KeepConnected: Yes';
        $header[] = 'Content-Type: application/json';
        $header[] = 'Content-Length: ' . strlen(json_encode($data));

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
        curl_setopt($ch, CURLOPT_URL, $url . $requestType);
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($data));
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);

        $result = curl_exec($ch);
        $result = json_decode($result, true);
        $error = curl_error($ch);
        curl_close($ch);
        if (!empty($result['error'])) {
            throw new \Exception($result['error']);
        } else if (!empty($error)) {
            throw new \Exception($error);
        } else {
            $session = $result;
            return isset($session['dataset'][$dataSetResultPos]) ? $session['dataset'][$dataSetResultPos] : null;
        }
    }
    
    public static function createDateProjectTask($dateFirst, $dateLast)
    {
        if($dateFirst){
            return self::createDateProject($dateFirst);
        } else if($dateLast){
            return self::createDateProject($dateLast);
        } else {
            return new \DateTime();
        }
    }

    public static function createDateProject($date)
    {
        return strpos($date, '-') === false
            ? \DateTime::createFromFormat(Utilities::FORMAT_DATE_DEFAULT_ONLY_DATE,$date)
            : new \DateTime($date);
    }

    /**
     * @param $taskId
     * @return bool | Entities\Task
     */
    public static function getMainRelatedTaskByTaskId($taskId) {
        $entityManager = self::getEntityManager();
        $relatedTaskIds = $entityManager->getRepository(self::ENTITY_DIR . 'TaskRelationship')
            ->findRelatedTaskIds($taskId);
        if(!empty($relatedTaskIds)) {
            return $entityManager->getRepository(self::ENTITY_DIR . 'Task')->find($relatedTaskIds['MAIN_TASK']);
        }
        return false;
    }

    public function setLogRequest($message,$time) {
        $this->logRequest[] = array($message ,round($time,3));
    }

    public static function getLogRequest() {
        return InstanceProvider::getEnvironment()->logRequest;
    }

    public function setLog($message,$isStart = false, $index = 0) {
        if($isStart){
            $this->start[$index] = microtime(true);
        }else{
            $end = microtime(true);
            $time = $end - $this->start[$index] ;

            $this->setLogRequest($message,$time);
            $this->start[$index] = microtime(true);
        }
    }

    public function getTokenSmartInSession() 
    {
        return $this->session->get("tokenZmart") ? $this->session->get("tokenZmart") : null;
    }

    public function setTokenZmartInSession($token) 
    {
        $this->session->set("tokenZmart", $token);
    }

    public function removeTokenZmartInSession()
    {
        $this->session->remove("tokenZmart");
    }

    public function getTokenInZmart()
    {
        if($this->getTokenSmartInSession()) {
            $token = $this->getTokenSmartInSession();
        } else {
            $param = array(
                'CLIENT_ID' => self::CLIENT_ID_LIBRARIES,
                'CLIENT_SECRET' => self::CLIENT_SECRET_LIBRARIES
            );
            $token = InstanceProvider::getLibsUtilities()->callWebService(null, '/lib_requestToken', $param, 'token',
                self::CLIENT_SECRET_LIBRARIES, $this->zmartUrl, true);
            $token = $token[0];
            $this->setTokenZmartInSession($token);
        }

        return $token;
    }

    /** Function from DevOps to Unpait one array key
     * */

    public static function getArrayKeyUnpaired($array,$trim = true)
    {
        $pairArray = array();
        foreach ($array as $key => $element) {
            if ($key % 2) {
                $pairArray[] = $trim ? trim($element) : $element;
            }
        }
        return $pairArray;
    }
}